import express, { type Express, Request, Response } from "express";
import { createServer, type Server } from "http";
import { storage } from "./storage";
import { z } from "zod";
import multer from "multer";
import path from "path";
import fs from "fs";
import { 
  insertUserSchema, 
  insertListingSchema, 
  insertMessageSchema, 
  loginUserSchema, 
  registerUserSchema
} from "@shared/schema";
import { isValidFile, saveFile, getPublicUrl, deleteFile } from "./utils/fileUpload";
import bcrypt from "bcryptjs";
import session from "express-session";
import MemoryStore from "memorystore";

// Create session store
const SessionStore = MemoryStore(session);

// Setup multer for file uploads
const upload = multer({
  storage: multer.memoryStorage(),
  limits: {
    fileSize: 5 * 1024 * 1024, // 5MB
  },
});

export async function registerRoutes(app: Express): Promise<Server> {
  // Create uploads directory if it doesn't exist
  const uploadsDir = path.join(process.cwd(), "uploads");
  if (!fs.existsSync(uploadsDir)) {
    fs.mkdirSync(uploadsDir, { recursive: true });
  }

  // Serve static files from uploads directory
  app.use("/uploads", express.static(uploadsDir));

  // Configure sessions
  app.use(
    session({
      cookie: { maxAge: 86400000 }, // 24 hours
      store: new SessionStore({
        checkPeriod: 86400000, // prune expired entries every 24h
      }),
      resave: false,
      saveUninitialized: false,
      secret: process.env.SESSION_SECRET || "gwadacoin-secret",
    })
  );

  // Middleware to check if user is authenticated
  const isAuthenticated = (req: Request, res: Response, next: express.NextFunction) => {
    if (req.session.userId) {
      return next();
    }
    res.status(401).json({ message: "Vous devez être connecté pour accéder à cette ressource" });
  };

  // Auth routes
  app.post("/api/auth/register", async (req: Request, res: Response) => {
    try {
      const validatedData = registerUserSchema.parse(req.body);
      
      // Check if username or email already exists
      const existingUsername = await storage.getUserByUsername(validatedData.username);
      if (existingUsername) {
        return res.status(400).json({ message: "Ce nom d'utilisateur est déjà utilisé" });
      }
      
      const existingEmail = await storage.getUserByEmail(validatedData.email);
      if (existingEmail) {
        return res.status(400).json({ message: "Cet email est déjà utilisé" });
      }
      
      // Hash password
      const hashedPassword = await bcrypt.hash(validatedData.password, 10);
      
      // Remove confirmPassword field and add hashed password
      const { confirmPassword, ...userDataWithoutConfirmPw } = validatedData;
      const userData = {
        ...userDataWithoutConfirmPw,
        password: hashedPassword,
      };
      
      const user = await storage.createUser(userData);
      
      // Remove password from response
      const { password, ...userWithoutPassword } = user;
      
      // Set session
      req.session.userId = user.id;
      
      res.status(201).json({ user: userWithoutPassword });
    } catch (error) {
      if (error instanceof z.ZodError) {
        return res.status(400).json({ message: error.errors[0].message });
      }
      res.status(500).json({ message: "Une erreur est survenue lors de l'inscription" });
    }
  });

  app.post("/api/auth/login", async (req: Request, res: Response) => {
    try {
      const validatedData = loginUserSchema.parse(req.body);
      
      const user = await storage.getUserByUsername(validatedData.username);
      if (!user) {
        return res.status(401).json({ message: "Nom d'utilisateur ou mot de passe incorrect" });
      }
      
      const passwordMatch = await bcrypt.compare(validatedData.password, user.password);
      if (!passwordMatch) {
        return res.status(401).json({ message: "Nom d'utilisateur ou mot de passe incorrect" });
      }
      
      // Set session
      req.session.userId = user.id;
      
      // Remove password from response
      const { password, ...userWithoutPassword } = user;
      
      res.status(200).json({ user: userWithoutPassword });
    } catch (error) {
      if (error instanceof z.ZodError) {
        return res.status(400).json({ message: error.errors[0].message });
      }
      res.status(500).json({ message: "Une erreur est survenue lors de la connexion" });
    }
  });

  app.post("/api/auth/logout", (req: Request, res: Response) => {
    req.session.destroy((err) => {
      if (err) {
        return res.status(500).json({ message: "Une erreur est survenue lors de la déconnexion" });
      }
      res.clearCookie("connect.sid");
      res.status(200).json({ message: "Déconnexion réussie" });
    });
  });

  app.get("/api/auth/current-user", async (req: Request, res: Response) => {
    if (!req.session.userId) {
      return res.status(401).json({ message: "Utilisateur non connecté" });
    }
    
    const user = await storage.getUser(req.session.userId);
    if (!user) {
      req.session.destroy(() => {});
      return res.status(401).json({ message: "Utilisateur non trouvé" });
    }
    
    // Remove password from response
    const { password, ...userWithoutPassword } = user;
    
    res.status(200).json({ user: userWithoutPassword });
  });

  // Categories routes
  app.get("/api/categories", async (req: Request, res: Response) => {
    try {
      const categories = await storage.getCategories();
      res.status(200).json({ categories });
    } catch (error) {
      res.status(500).json({ message: "Une erreur est survenue lors de la récupération des catégories" });
    }
  });

  app.get("/api/categories/:slug", async (req: Request, res: Response) => {
    try {
      const { slug } = req.params;
      const category = await storage.getCategoryBySlug(slug);
      
      if (!category) {
        return res.status(404).json({ message: "Catégorie non trouvée" });
      }
      
      res.status(200).json({ category });
    } catch (error) {
      res.status(500).json({ message: "Une erreur est survenue lors de la récupération de la catégorie" });
    }
  });

  // Listings routes
  app.get("/api/listings", async (req: Request, res: Response) => {
    try {
      const limit = req.query.limit ? parseInt(req.query.limit as string) : 20;
      const offset = req.query.offset ? parseInt(req.query.offset as string) : 0;
      const categoryId = req.query.categoryId ? parseInt(req.query.categoryId as string) : undefined;
      const searchTerm = req.query.search as string | undefined;
      const minPrice = req.query.minPrice ? parseFloat(req.query.minPrice as string) : undefined;
      const maxPrice = req.query.maxPrice ? parseFloat(req.query.maxPrice as string) : undefined;
      const location = req.query.location as string | undefined;
      const sortBy = req.query.sortBy as string | undefined;
      const sortOrder = req.query.sortOrder as 'asc' | 'desc' | undefined;
      
      const { listings, total } = await storage.getListings({
        limit,
        offset,
        categoryId,
        searchTerm,
        minPrice,
        maxPrice,
        location,
        sortBy,
        sortOrder
      });
      
      // Get the primary image for each listing
      const listingsWithImages = await Promise.all(
        listings.map(async (listing) => {
          const images = await storage.getListingImages(listing.id);
          const primaryImage = images.find(img => img.isPrimary) || images[0];
          return {
            ...listing,
            primaryImage: primaryImage ? getPublicUrl(primaryImage.imageUrl) : null,
          };
        })
      );
      
      res.status(200).json({ 
        listings: listingsWithImages, 
        pagination: {
          total,
          limit,
          offset,
          hasMore: offset + limit < total
        } 
      });
    } catch (error) {
      res.status(500).json({ message: "Une erreur est survenue lors de la récupération des annonces" });
    }
  });

  app.get("/api/listings/featured", async (req: Request, res: Response) => {
    try {
      const { listings } = await storage.getListings({ limit: 8 });
      
      // Get the primary image for each listing
      const listingsWithImages = await Promise.all(
        listings.map(async (listing) => {
          const images = await storage.getListingImages(listing.id);
          const primaryImage = images.find(img => img.isPrimary) || images[0];
          return {
            ...listing,
            primaryImage: primaryImage ? getPublicUrl(primaryImage.imageUrl) : null,
          };
        })
      );
      
      res.status(200).json({ listings: listingsWithImages });
    } catch (error) {
      res.status(500).json({ message: "Une erreur est survenue lors de la récupération des annonces à la une" });
    }
  });

  app.get("/api/listings/user/:userId", isAuthenticated, async (req: Request, res: Response) => {
    try {
      const userId = parseInt(req.params.userId);
      
      // Users should only be able to see their own listings unless an admin feature is added later
      if (req.session.userId !== userId) {
        return res.status(403).json({ message: "Vous n'êtes pas autorisé à voir ces annonces" });
      }
      
      const { listings } = await storage.getListings({ userId });
      
      // Get the primary image for each listing
      const listingsWithImages = await Promise.all(
        listings.map(async (listing) => {
          const images = await storage.getListingImages(listing.id);
          const primaryImage = images.find(img => img.isPrimary) || images[0];
          return {
            ...listing,
            primaryImage: primaryImage ? getPublicUrl(primaryImage.imageUrl) : null,
          };
        })
      );
      
      res.status(200).json({ listings: listingsWithImages });
    } catch (error) {
      res.status(500).json({ message: "Une erreur est survenue lors de la récupération des annonces" });
    }
  });

  app.get("/api/listings/:id", async (req: Request, res: Response) => {
    try {
      const id = parseInt(req.params.id);
      const listing = await storage.getListing(id);
      
      if (!listing) {
        return res.status(404).json({ message: "Annonce non trouvée" });
      }
      
      // Get images for the listing
      const images = await storage.getListingImages(id);
      const imagesWithUrls = images.map(img => ({
        ...img,
        imageUrl: getPublicUrl(img.imageUrl)
      }));
      
      // Get seller information
      const seller = await storage.getUser(listing.userId);
      if (!seller) {
        return res.status(404).json({ message: "Vendeur non trouvé" });
      }
      
      // Remove password from seller
      const { password, ...sellerWithoutPassword } = seller;
      
      // Get category information
      const category = await storage.getCategory(listing.categoryId);
      
      // Check if the listing is in the current user's favorites
      let isFavorite = false;
      if (req.session.userId) {
        isFavorite = await storage.isFavorite(req.session.userId, id);
      }
      
      res.status(200).json({ 
        listing, 
        images: imagesWithUrls, 
        seller: sellerWithoutPassword,
        category,
        isFavorite
      });
    } catch (error) {
      res.status(500).json({ message: "Une erreur est survenue lors de la récupération de l'annonce" });
    }
  });

  app.post("/api/listings", isAuthenticated, async (req: Request, res: Response) => {
    try {
      const validatedData = insertListingSchema.parse({
        ...req.body,
        userId: req.session.userId
      });
      
      const listing = await storage.createListing(validatedData);
      
      res.status(201).json({ listing });
    } catch (error) {
      if (error instanceof z.ZodError) {
        return res.status(400).json({ message: error.errors[0].message });
      }
      res.status(500).json({ message: "Une erreur est survenue lors de la création de l'annonce" });
    }
  });

  app.patch("/api/listings/:id", isAuthenticated, async (req: Request, res: Response) => {
    try {
      const id = parseInt(req.params.id);
      const listing = await storage.getListing(id);
      
      if (!listing) {
        return res.status(404).json({ message: "Annonce non trouvée" });
      }
      
      // Verify ownership
      if (listing.userId !== req.session.userId) {
        return res.status(403).json({ message: "Vous n'êtes pas autorisé à modifier cette annonce" });
      }
      
      const validatedData = insertListingSchema.partial().parse(req.body);
      
      const updatedListing = await storage.updateListing(id, validatedData);
      
      res.status(200).json({ listing: updatedListing });
    } catch (error) {
      if (error instanceof z.ZodError) {
        return res.status(400).json({ message: error.errors[0].message });
      }
      res.status(500).json({ message: "Une erreur est survenue lors de la mise à jour de l'annonce" });
    }
  });

  app.delete("/api/listings/:id", isAuthenticated, async (req: Request, res: Response) => {
    try {
      const id = parseInt(req.params.id);
      const listing = await storage.getListing(id);
      
      if (!listing) {
        return res.status(404).json({ message: "Annonce non trouvée" });
      }
      
      // Verify ownership
      if (listing.userId !== req.session.userId) {
        return res.status(403).json({ message: "Vous n'êtes pas autorisé à supprimer cette annonce" });
      }
      
      // Delete images first
      const images = await storage.getListingImages(id);
      for (const image of images) {
        await storage.deleteListingImage(image.id);
        deleteFile(image.imageUrl);
      }
      
      await storage.deleteListing(id);
      
      res.status(200).json({ message: "Annonce supprimée avec succès" });
    } catch (error) {
      res.status(500).json({ message: "Une erreur est survenue lors de la suppression de l'annonce" });
    }
  });

  // Image upload routes
  app.post("/api/listings/:id/images", isAuthenticated, upload.single('image'), async (req: Request, res: Response) => {
    try {
      const listingId = parseInt(req.params.id);
      const listing = await storage.getListing(listingId);
      
      if (!listing) {
        return res.status(404).json({ message: "Annonce non trouvée" });
      }
      
      // Verify ownership
      if (listing.userId !== req.session.userId) {
        return res.status(403).json({ message: "Vous n'êtes pas autorisé à modifier cette annonce" });
      }
      
      if (!req.file) {
        return res.status(400).json({ message: "Aucune image téléchargée" });
      }
      
      // Validate file
      if (!isValidFile(req.file.mimetype, req.file.size)) {
        return res.status(400).json({ message: "Type de fichier ou taille non valide" });
      }
      
      // Save file
      const fileName = await saveFile(req.file.buffer, req.file.mimetype);
      
      // Create listing image entry
      const listingImage = await storage.createListingImage({
        listingId,
        imageUrl: fileName,
        isPrimary: false
      });
      
      res.status(201).json({ 
        image: {
          ...listingImage,
          imageUrl: getPublicUrl(fileName)
        } 
      });
    } catch (error) {
      res.status(500).json({ message: "Une erreur est survenue lors du téléchargement de l'image" });
    }
  });

  app.delete("/api/listings/:listingId/images/:imageId", isAuthenticated, async (req: Request, res: Response) => {
    try {
      const listingId = parseInt(req.params.listingId);
      const imageId = parseInt(req.params.imageId);
      
      const listing = await storage.getListing(listingId);
      
      if (!listing) {
        return res.status(404).json({ message: "Annonce non trouvée" });
      }
      
      // Verify ownership
      if (listing.userId !== req.session.userId) {
        return res.status(403).json({ message: "Vous n'êtes pas autorisé à modifier cette annonce" });
      }
      
      const images = await storage.getListingImages(listingId);
      const image = images.find(img => img.id === imageId);
      
      if (!image) {
        return res.status(404).json({ message: "Image non trouvée" });
      }
      
      // Delete the file
      deleteFile(image.imageUrl);
      
      // Delete the database entry
      await storage.deleteListingImage(imageId);
      
      res.status(200).json({ message: "Image supprimée avec succès" });
    } catch (error) {
      res.status(500).json({ message: "Une erreur est survenue lors de la suppression de l'image" });
    }
  });

  app.post("/api/listings/:listingId/images/:imageId/primary", isAuthenticated, async (req: Request, res: Response) => {
    try {
      const listingId = parseInt(req.params.listingId);
      const imageId = parseInt(req.params.imageId);
      
      const listing = await storage.getListing(listingId);
      
      if (!listing) {
        return res.status(404).json({ message: "Annonce non trouvée" });
      }
      
      // Verify ownership
      if (listing.userId !== req.session.userId) {
        return res.status(403).json({ message: "Vous n'êtes pas autorisé à modifier cette annonce" });
      }
      
      const result = await storage.setPrimaryListingImage(listingId, imageId);
      
      if (!result) {
        return res.status(404).json({ message: "Image non trouvée" });
      }
      
      res.status(200).json({ message: "Image principale définie avec succès" });
    } catch (error) {
      res.status(500).json({ message: "Une erreur est survenue lors de la définition de l'image principale" });
    }
  });

  // Message routes
  app.get("/api/messages", isAuthenticated, async (req: Request, res: Response) => {
    try {
      const userId = req.session.userId!;
      const listingId = req.query.listingId ? parseInt(req.query.listingId as string) : undefined;
      
      const messages = await storage.getMessages(userId, listingId);
      
      res.status(200).json({ messages });
    } catch (error) {
      res.status(500).json({ message: "Une erreur est survenue lors de la récupération des messages" });
    }
  });

  app.post("/api/messages", isAuthenticated, async (req: Request, res: Response) => {
    try {
      const senderId = req.session.userId!;
      
      const validatedData = insertMessageSchema.parse({
        ...req.body,
        senderId
      });
      
      // Verify that the listing exists
      const listing = await storage.getListing(validatedData.listingId);
      if (!listing) {
        return res.status(404).json({ message: "Annonce non trouvée" });
      }
      
      // Users cannot message themselves
      if (senderId === validatedData.receiverId) {
        return res.status(400).json({ message: "Vous ne pouvez pas vous envoyer un message à vous-même" });
      }
      
      const message = await storage.createMessage(validatedData);
      
      res.status(201).json({ message });
    } catch (error) {
      if (error instanceof z.ZodError) {
        return res.status(400).json({ message: error.errors[0].message });
      }
      res.status(500).json({ message: "Une erreur est survenue lors de l'envoi du message" });
    }
  });

  // Mark message as read
  app.patch("/api/messages/:id/read", isAuthenticated, async (req: Request, res: Response) => {
    try {
      const id = parseInt(req.params.id);
      const result = await storage.markMessageAsRead(id);
      
      if (!result) {
        return res.status(404).json({ message: "Message non trouvé" });
      }
      
      res.status(200).json({ message: "Message marqué comme lu" });
    } catch (error) {
      res.status(500).json({ message: "Une erreur est survenue lors du marquage du message comme lu" });
    }
  });

  // Favorites routes
  app.get("/api/favorites", isAuthenticated, async (req: Request, res: Response) => {
    try {
      const userId = req.session.userId!;
      const listings = await storage.getUserFavorites(userId);
      
      // Get primary images
      const listingsWithImages = await Promise.all(
        listings.map(async (listing) => {
          const images = await storage.getListingImages(listing.id);
          const primaryImage = images.find(img => img.isPrimary) || images[0];
          return {
            ...listing,
            primaryImage: primaryImage ? getPublicUrl(primaryImage.imageUrl) : null,
          };
        })
      );
      
      res.status(200).json({ listings: listingsWithImages });
    } catch (error) {
      res.status(500).json({ message: "Une erreur est survenue lors de la récupération des favoris" });
    }
  });

  app.post("/api/favorites", isAuthenticated, async (req: Request, res: Response) => {
    try {
      const userId = req.session.userId!;
      const { listingId } = req.body;
      
      if (!listingId) {
        return res.status(400).json({ message: "ID d'annonce manquant" });
      }
      
      // Verify that the listing exists
      const listing = await storage.getListing(parseInt(listingId));
      if (!listing) {
        return res.status(404).json({ message: "Annonce non trouvée" });
      }
      
      const favorite = await storage.addFavorite({
        userId,
        listingId: parseInt(listingId)
      });
      
      res.status(201).json({ favorite });
    } catch (error) {
      res.status(500).json({ message: "Une erreur est survenue lors de l'ajout aux favoris" });
    }
  });

  app.delete("/api/favorites/:listingId", isAuthenticated, async (req: Request, res: Response) => {
    try {
      const userId = req.session.userId!;
      const listingId = parseInt(req.params.listingId);
      
      const result = await storage.removeFavorite(userId, listingId);
      
      if (!result) {
        return res.status(404).json({ message: "Favori non trouvé" });
      }
      
      res.status(200).json({ message: "Favori supprimé avec succès" });
    } catch (error) {
      res.status(500).json({ message: "Une erreur est survenue lors de la suppression du favori" });
    }
  });

  const httpServer = createServer(app);
  return httpServer;
}
